#!BPY

""" Registration info for Blender menus:
Name: 'Nexuiz weapon exporter...'
Blender: 235
Group: 'Export'
Submenu: 'Screen weapon Model (zym)' zym_sel
Submenu: 'Ground weapon Model (md3)' md3g_sel
Submenu: 'Player weapon Model (md3)' md3_sel
Submenu: 'All weapon Models (zym & md3)' all_sel
Submenu: 'Dump MD3' md3d_sel
Submenu: 'Dump rotating MD3' md3dr_sel
Submenu: 'Dump SMD' smd_sel
Submenu: 'Dump ZYM' zymd_sel

Tip: 'Export Nexuiz weapon models. Use Dump options to disable internal transformations.'
"""

__author__ = "Allister Brizan"
__url__ = ("blender", "elysiun", "Nexuiz, http://alientrap.org/nexuiz/",
	"PLib 3d gaming lib, ")
__version__ = "2.35 12/21/04"

__bpydoc__ = """\
This script exports Blender meshes to Nexuiz weapon models.

Supported:<br>
        Hand weapon models (what the enemy/teammate is carrying)
        Ground weapon models (spinning icon to pick up)
        Screen weapon models (first person view)

Known issues:<br>
    Windows only. :(

Notes: <br>SMD export is a copy of the script by  Pelle Johnsen.<br>
md3 export is a copy of the script by the XreaL team.
    
"""

# $Id$
#
# --------------------------------------------------------------------------
# smd_export version 2.35.2
# Program versions: Blender 2.35 and HalfLife2
# --------------------------------------------------------------------------
# ***** BEGIN GPL LICENSE BLOCK *****
#
# Copyright (C) 2004: Pelle Johnsen, pelle.johnsen _at_ mail.dk
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****
# --------------------------------------------------------------------------


# -------------------------------------------------------------------------
#Config MD3 transforms

# Player model scale
md3scalefactor = 4
#screen model scale, must be in quotes
zymscalefactor = '4.4'
#Ground model scale and translation
md3gscalefactor = 5.5
md3goffsetX = -7.1772/2
md3goffsetY = -0.1895
md3goffsetZ = 1

#end Config
# -------------------------------------------------------------------------




import Blender
from Blender import *
from Blender import sys as SYS
from Blender.Draw import *
from Blender.BGL import *
from Blender.Window import *
import sys, os, os.path, struct, string, math

import types

import md3
from md3 import *

import q_math
from q_math import *

import q_shared
from q_shared import *

Ysep = SYS.sep
Ypath = SYS.dirname(SYS.progname)+Ysep
Bver = Blender.Get("version")
Yscriptspath = Blender.Get("scriptsdir")

ARG = __script__['arg']

# put menu selection in vars
zym_sel = (ARG == 'zym_sel')
zymd_sel = (ARG == 'zymd_sel')
all_sel = (ARG == 'all_sel')
smd_sel = (ARG == 'smd_sel')
md3_sel = (ARG == 'md3_sel')
md3g_sel = (ARG == 'md3g_sel')
md3d_sel = (ARG == 'md3d_sel')
md3dr_sel = (ARG == 'md3dr_sel')

def ApplyTransform(vert, matrix):
	return vert * matrix


def UpdateFrameBounds(v, f):
	for i in range(0, 3):
		f.mins[i] = min(v[i], f.mins[i])
	for i in range(0, 3):
		f.maxs[i] = max(v[i], f.maxs[i])


def UpdateFrameRadius(f):
	f.radius = RadiusFromBounds(f.mins, f.maxs)


def ProcessSurface(scene, blenderObject, md3, pathName, modelName):
	# because md3 doesnt suppoort faceUVs like blender, we need to duplicate
	# any vertex that has multiple uv coords

	vertDict = {}
	indexDict = {} # maps a vertex index to the revised index after duplicating to account for uv
	vertList = [] # list of vertices ordered by revised index
	numVerts = 0
	uvList = [] # list of tex coords ordered by revised index
	faceList = [] # list of faces (they index into vertList)
	numFaces = 0

	scene.makeCurrent()
	Blender.Set("curframe", 1)
	Blender.Window.Redraw()

	# get the object (not just name) and the Mesh, not NMesh
	mesh = blenderObject.getData(False, True)

	if md3g_sel:        
        	#Apply transform for centre-of-rotation
        	for v in mesh.verts:
                        v.co.x+=md3goffsetX*md3gscalefactor
                        v.co.y+=md3goffsetY*md3gscalefactor
                        v.co.z+=md3goffsetZ*md3gscalefactor


	matrix = blenderObject.getMatrix('worldspace')

	surf = md3Surface()
	surf.numFrames = md3.numFrames
	surf.name = blenderObject.getName()
	surf.ident = MD3_IDENT
	
	# create shader for surface
	surf.shaders.append(md3Shader())
	surf.numShaders += 1
	surf.shaders[0].index = 0
	
	# :P
	#shaderpath=Blender.Draw.PupStrInput("shader path for "+blenderObject.name+":", "", MAX_QPATH )
	shaderpath="textures/"
	if 	shaderpath == "" :
		if not mesh.materials:
			surf.shaders[0].name = pathName + blenderObject.name
		else:
			surf.shaders[0].name = pathName + mesh.materials[0].name
	else:
		if not mesh.materials:
			surf.shaders[0].name = shaderpath + blenderObject.name
		else:
			surf.shaders[0].name = shaderpath + mesh.materials[0].name

	# process each face in the mesh
	for face in mesh.faces:
		
		tris_in_this_face = []  #to handle quads and up...
		
		# this makes a list of indices for each tri in this face. a quad will be [[0,1,1],[0,2,3]]
		for vi in range(1, len(face.v)-1):
			tris_in_this_face.append([0, vi, vi + 1])
		
		# loop across each tri in the face, then each vertex in the tri
		for this_tri in tris_in_this_face:
			numFaces += 1
			tri = md3Triangle()
			tri_ind = 0
			for i in this_tri:
				# get the vertex index, coords and uv coords
				index = face.v[i].index
				v = face.v[i].co
				if mesh.faceUV == True:
					uv = (face.uv[i][0], face.uv[i][1])
				elif mesh.vertexUV:
					uv = (face.v[i].uvco[0], face.v[i].uvco[1])
				else:
					uv = (0.0, 0.0) # handle case with no tex coords	

				
				if vertDict.has_key((index, uv)):
					# if we've seen this exact vertex before, simply add it
					# to the tris list of vertex indices
					tri.indexes[tri_ind] = vertDict[(index, uv)]
				else:
					# havent seen this tri before 
					# (or its uv coord is different, so we need to duplicate it)
					
					vertDict[(index, uv)] = numVerts
					
					# put the uv coord into the list
					# (uv coord are directly related to each vertex)
					tex = md3TexCoord()
					tex.u = uv[0]
					tex.v = uv[1]
					uvList.append(tex)

					tri.indexes[tri_ind] = numVerts

					# now because we have created a new index, 
					# we need a way to link it to the index that
					# blender returns for NMVert.index
					if indexDict.has_key(index):
						# already there - each of the entries against 
						# this key represents  the same vertex with a
						# different uv value
						ilist = indexDict[index]
						ilist.append(numVerts)
						indexDict[index] = ilist
					else:
						# this is a new one
						indexDict[index] = [numVerts]

					numVerts += 1
				tri_ind +=1
			faceList.append(tri)

	# we're done with faces and uv coords
	for t in uvList:
		surf.uv.append(t)

	for f in faceList:
		surf.triangles.append(f)

	surf.numTriangles = len(faceList)
	surf.numVerts = numVerts

	# now vertices are stored as frames -
	# all vertices for frame 1, all vertices for frame 2...., all vertices for frame n
	# so we need to iterate across blender's frames, and copy out each vertex
	for	frameNum in range(1, md3.numFrames + 1):
		Blender.Set("curframe", frameNum)
		Blender.Window.Redraw()

		m = NMesh.GetRawFromObject(blenderObject.name)

		vlist = [0] * numVerts
		for vertex in m.verts:
			try:
				vindices = indexDict[vertex.index]
			except:
				#log.warning("Found a vertex in %s that is not part of a face", blenderObject.name)
                                print "Found a vertex in %s that is not part of a face"
				continue

			vTx = ApplyTransform(vertex.co, matrix)
			nTx = ApplyTransform(vertex.no, matrix)
			UpdateFrameBounds(vTx, md3.frames[frameNum - 1])
			vert = md3Vert()
			#vert.xyz = vertex.co[0:3]
			#vert.normal = vert.Encode(vertex.no[0:3])
			vert.xyz = vTx[0:3]
			vert.normal = vert.Encode(nTx[0:3])
			for ind in vindices:  # apply the position to all the duplicated vertices
				vlist[ind] = vert

		UpdateFrameRadius(md3.frames[frameNum - 1])

		for vl in vlist:
			surf.verts.append(vl)

	md3.surfaces.append(surf)
	md3.numSurfaces += 1

	if md3g_sel:        
        	#Undo transform for centre-of-rotation
        	for v in mesh.verts:
                        v.co.x-=md3goffsetX*md3gscalefactor
                        v.co.y-=md3goffsetY*md3gscalefactor
                        v.co.z-=md3goffsetZ*md3gscalefactor


def MD3Export(fileName):
	
	pathName = StripGamePath(StripModel(fileName))
	
	modelName = StripExtension(StripPath(fileName))
	
	md3 = md3Object()
	md3.ident = MD3_IDENT
	md3.version = MD3_VERSION

	tagList = []
	
	# get the scene
	scene = Blender.Scene.getCurrent()
	context = scene.getRenderingContext()

	scene.makeCurrent()
	md3.numFrames = Blender.Get("curframe")
	Blender.Set("curframe", 1)

	# create a bunch of blank frames, they'll be filled in by 'ProcessSurface'
	for i in range(1, md3.numFrames + 1):
		frame = md3Frame()
		frame.name = "frame_" + str(i)
		md3.frames.append(frame)

	# export all selected objects
	objlist = Blender.Object.GetSelected()

	# process each object for the export
	for obj in objlist:
		# check if it's a mesh object
		if obj.getType() == "Mesh":
                        if md3_sel: 
                                #Apply MD3 player weapon model scale
                                obj.SizeX = obj.SizeX * md3scalefactor
                                obj.SizeY = obj.SizeY * md3scalefactor
                                obj.SizeZ = obj.SizeZ * md3scalefactor

                        if md3g_sel:
                                #Apply MD3 player ground model offsets before scale
                                obj.SizeX = obj.SizeX * md3gscalefactor
                                obj.SizeY = obj.SizeY * md3gscalefactor
                                obj.SizeZ = obj.SizeZ * md3gscalefactor
                        
			if len(md3.surfaces) != MD3_MAX_SURFACES:
				ProcessSurface(scene, obj, md3, pathName, modelName)

			if md3_sel:
                                #Undo MD3 player weapon model scale
                                obj.SizeX = obj.SizeX / md3scalefactor
                                obj.SizeY = obj.SizeY / md3scalefactor
                                obj.SizeZ = obj.SizeZ / md3scalefactor

                        if md3g_sel: 
                                #Undo MD3 player ground model scale
                                obj.SizeX = obj.SizeX / md3gscalefactor
                                obj.SizeY = obj.SizeY / md3gscalefactor
                                obj.SizeZ = obj.SizeZ / md3gscalefactor                                
                                me = obj.getData(mesh=1)

                                        
		elif obj.getType() == "Empty":   # for tags, we just put em in a list so we can process them all together
			if obj.name[0:4] == "tag_":
				tagList.append(obj)
				md3.numTags += 1
	
	# work out the transforms for the tags for each frame of the export
	for i in range(1, md3.numFrames + 1):
		
		# needed to update IPO's value, but probably not the best way for that...
		scene.makeCurrent()
		Blender.Set("curframe", i)
		Blender.Window.Redraw()
		for tag in tagList:
			t = md3Tag()
			matrix = tag.getMatrix('worldspace')
			t.origin[0] = matrix[3][0]
			t.origin[1] = matrix[3][1]
			t.origin[2] = matrix[3][2]
				
			t.axis[0] = matrix[0][0]
			t.axis[1] = matrix[0][1]
			t.axis[2] = matrix[0][2]
				
			t.axis[3] = matrix[1][0]
			t.axis[4] = matrix[1][1]
			t.axis[5] = matrix[1][2]
				
			t.axis[6] = matrix[2][0]
			t.axis[7] = matrix[2][1]
			t.axis[8] = matrix[2][2]
			t.name = tag.name
			#t.Dump(log)
			md3.tags.append(t)

	# export!
	file = open(fileName, "wb")
	md3.Save(file)
	file.close()
#	md3.Dump(log)






def writeTri(file, object, face, verts, centre):
	mesh = object.getData()

	mesh.materials[0].name

	if mesh.hasFaceUV():
                file.write("%s\n" %mesh.materials[0].name)
	else:
		file.write("crylink\n")
		
	for vi in verts:
		v = face.v[vi]
		# v.co(ordinates)
		vec = Blender.Mathutils.Vector([v.co[0], v.co[1], v.co[2], 1])
		vec = Blender.Mathutils.VecMultMat(vec, object.getMatrix('worldspace'))
		
		file.write("0 %f %f %f " % (vec.x - centre[0], vec.y - centre[1], vec.z - centre[2]))
			
		# v.no are normals... subtract object orientation?
		if face.smooth:
			vec = Blender.Mathutils.Vector([v.no[0], v.no[1], v.no[2], 1])
			vec = Blender.Mathutils.VecMultMat(vec, object.getMatrix('worldspace'))
			file.write("%f %f %f " % (vec.x, vec.y, vec.z))
		else:
			vec = Blender.Mathutils.Vector([face.no[0], face.no[1], face.no[2], 1])
			vec = Blender.Mathutils.VecMultMat(vec, object.getMatrix('worldspace'))
			file.write("%f %f %f " % (vec.x, vec.y, vec.z))
		
		if mesh.hasFaceUV():
			file.write("%f %f\n" % (face.uv[vi][0], face.uv[vi][1]))
		else:
			file.write("%f %f\n" % (0, 0))

class SMDExport:

	def __init__(self, filename):
		print 'Trying SMDExport...'

		objects = Blender.Object.GetSelected()
	
		try:
			file = open(filename, 'w')
		except IOError, (errno, strerror):
			errmsg = "IOError #%s" % errno
			errmsg = errmsg + "%t|" + strerror
			Blender.Draw.PupMenu(errmsg)
			return None
		file.write("version 1\n")
		file.write("nodes\n")
		file.write("0 \"Bone01\" -1\n")
		file.write("end\n")
		file.write("skeleton\n")
		file.write("time 0\n")
		file.write("0 %f %f %f %f %f %f\n" % (0, 0, 0, 0, 0, 0))
		file.write("end\n")
		file.write("triangles\n")

		noUVWarned = False
		centre = (0, 0, 0)
		for object in objects:
			if object.getType() == 'Empty':
				centre = object.loc
				print 'found centre %f, %f, %f' % centre
	
				
		
		for object in objects:
			if object.getType() != 'Mesh': continue

			if zym_sel:
                                #Rotate before SMD export (for zym)
        			object.RotX -= 3.14159/2

			print 'Processing Mesh:' + object.name
			mesh = object.getData()

			if True or mesh.hasFaceUV():
				faces = mesh.faces
				for face in faces:
					writeTri(file, object, face, [0, 1, 2], centre)
					if len(face.v) == 4:
						writeTri(file, object, face, [0, 2, 3], centre)

			if zym_sel:
        			#UnRotate After SMD export (for zym)
                                object.RotX += 3.14159/2

		
	
		file.write("end\n")
		file.close()

		print "Done. Saved to %s\n" % filename
		
			

# -- end of class SMDExport

def makeALL_callback(filename):
        global md3g_sel
        global md3_sel
        global zym_sel

	pathName = StripGamePath(StripModel(filename))
	modelName = StripExtension(StripPath(filename))

	print 'Exporting ' + pathName + 'g_' + modelName + '.md3'
	md3g_sel = True
	makeMD3g_callback(pathName + 'g_' + modelName + '.md3')
	md3g_sel = False

	print 'Exporting ' + pathName + 'v_' + modelName + '.md3'
	md3_sel = True
	makeMD3_callback(pathName + 'v_' + modelName + '.md3')
	md3_sel = False
	
	print 'Exporting ' + pathName + 'w_' + modelName + '.zym'
	zym_sel = True
	makeZYM_callback(pathName + 'w_' + modelName + '.zym')
	zym_sel = False

def makeSMD_callback(filename):
	if filename.find('.smd', -4) <= 0: filename += '.smd'
	test = SMDExport(filename)

def makeMD3_callback(filename):
	if filename.find('.md3', -4) <= 0: filename += '.md3'
	MD3Export(filename)

def makeMD3g_callback(filename):
	if filename.find('.md3', -4) <= 0: filename += '.md3'
	zmodelcommand =  Ypath + '.blender\scripts\\nexwe\\modtool.exe --flags 8 '+ filename
	zmodelcommand = zmodelcommand.replace ("\\" ,"/")
	MD3Export(filename)
	os.system(zmodelcommand)

def makeZYM_callback(filename):
	if filename.find('.zym', -4) <= 0: filename += '.zym'
	#Make zmodel configuration file
	try:
		file = open(Yscriptspath + "\\nexwe\config.txt", 'w')
	except IOError, (errno, strerror):
		errmsg = "IOError #%s" % errno
		errmsg = errmsg + "%t|" + strerror
		Blender.Draw.PupMenu(errmsg)
		return None
	file.write('output "' +filename+ '"\n')
	file.write('texturedir textures\n')
	file.write('scale ' + zymscalefactor + '\n')
	file.write('origin 0 0 0\n')
	filebuffer = 'mesh "' + Yscriptspath + '\\nexwe\\temp.smd"\n'
	filebuffer += 'scene "' + Yscriptspath + '\\nexwe\\fire.smd" fps 8\n'
	filebuffer += 'scene "' + Yscriptspath + '\\nexwe\\fire2.smd" fps 7\n'
	filebuffer += 'scene "' + Yscriptspath + '\\nexwe\idle.smd" fps 1\n'
	filebuffer += 'scene "' + Yscriptspath + '\\nexwe\\run.smd" fps 6\n'

	file.write (filebuffer)
	file.close()

	zmodelcommand =  Ypath + '.blender\scripts\\nexwe\\zmodel.exe '+ Ypath + '.blender\scripts\\nexwe\\config.txt'
	zmodelcommand = zmodelcommand.replace ("\\" ,"/")

        #Export temporary mesh and call zmodel
	test = SMDExport(Yscriptspath + '\\nexwe\\temp.smd')
	if test:
                print "Zmodel configured, attempting to run.."
                print " "
                os.system(zmodelcommand)
                print " "
                print "Success.."
        else:
        	print Yscriptspath + '\\nexwe\\temp.smd not made'


# Main script logic


if ARG == 'config':
	exit
else:
	if smd_sel:
		fname = Blender.sys.makename(ext=".smd")
		Blender.Window.FileSelector(makeSMD_callback, "Export SMD", fname)
	if zym_sel or zymd_sel:
		fname = Blender.sys.makename(ext=".zym")	
		Blender.Window.FileSelector(makeZYM_callback, "Export ZYM", fname)		
	if md3_sel or md3d_sel:
		fname = Blender.sys.makename(ext=".md3")	
		Blender.Window.FileSelector(makeMD3_callback, "Export MD3", fname)
	if md3g_sel or md3dr_sel:
		fname = Blender.sys.makename(ext=".md3")	
		Blender.Window.FileSelector(makeMD3g_callback, "Export MD3", fname)
	if all_sel:
		fname = Blender.sys.makename(ext="")
		Blender.Window.FileSelector(makeALL_callback, "Export ALL (no ext.)", fname)		

