import struct, math

def stripnull(s, l):
	n = 0
	while( n < l and ord(s[n]) != 0):
		n = n + 1
	return s[0:n]

def vertorg(relativeorigin, m):
	out = [0,0,0]
	out[0] = relativeorigin[0] * m[0] + relativeorigin[1] * m[1] + relativeorigin[2] * m[ 2] + m[ 3]
	out[1] = relativeorigin[0] * m[4] + relativeorigin[1] * m[5] + relativeorigin[2] * m[ 6] + m[ 7]
	out[2] = relativeorigin[0] * m[8] + relativeorigin[1] * m[9] + relativeorigin[2] * m[10] + m[11]
	return out

def poseloc(pose):
	return [ pose[3], pose[7], pose[11] ]

def posex(pose):
	return [ pose[0], pose[4], pose[8] ]

def posey(pose):
	return [ pose[1], pose[5], pose[9] ]

def posez(pose):
	return [ pose[2], pose[6], pose[10] ]

def poserot(pose):
	return [pose[0],pose[1],pose[2]],[pose[4],pose[5],pose[6]],[pose[8],pose[9],pose[10]]

def contrans(in1, in2):
	out = [0,0,0,0, 0,0,0,0, 0,0,0,0]
	out[0*4+0] = in1[0*4+0] * in2[0*4+0] + in1[0*4+1] * in2[1*4+0] + in1[0*4+2] * in2[2*4+0]
	out[0*4+1] = in1[0*4+0] * in2[0*4+1] + in1[0*4+1] * in2[1*4+1] + in1[0*4+2] * in2[2*4+1]
	out[0*4+2] = in1[0*4+0] * in2[0*4+2] + in1[0*4+1] * in2[1*4+2] + in1[0*4+2] * in2[2*4+2]
	out[0*4+3] = in1[0*4+0] * in2[0*4+3] + in1[0*4+1] * in2[1*4+3] + in1[0*4+2] * in2[2*4+3] + in1[0*4+3]
	out[1*4+0] = in1[1*4+0] * in2[0*4+0] + in1[1*4+1] * in2[1*4+0] + in1[1*4+2] * in2[2*4+0]
	out[1*4+1] = in1[1*4+0] * in2[0*4+1] + in1[1*4+1] * in2[1*4+1] + in1[1*4+2] * in2[2*4+1]
	out[1*4+2] = in1[1*4+0] * in2[0*4+2] + in1[1*4+1] * in2[1*4+2] + in1[1*4+2] * in2[2*4+2]
	out[1*4+3] = in1[1*4+0] * in2[0*4+3] + in1[1*4+1] * in2[1*4+3] + in1[1*4+2] * in2[2*4+3] + in1[1*4+3]
	out[2*4+0] = in1[2*4+0] * in2[0*4+0] + in1[2*4+1] * in2[1*4+0] + in1[2*4+2] * in2[2*4+0]
	out[2*4+1] = in1[2*4+0] * in2[0*4+1] + in1[2*4+1] * in2[1*4+1] + in1[2*4+2] * in2[2*4+1]
	out[2*4+2] = in1[2*4+0] * in2[0*4+2] + in1[2*4+1] * in2[1*4+2] + in1[2*4+2] * in2[2*4+2]
	out[2*4+3] = in1[2*4+0] * in2[0*4+3] + in1[2*4+1] * in2[1*4+3] + in1[2*4+2] * in2[2*4+3] + in1[2*4+3]
	return out


class zymlump(object):
	__slots__ = 'start', 'length', 'binaryFormat'
	
	binaryFormat = "!ii"
	
	def __init__(self):
		self.start = 0
		self.length = 0
		
	def getsize(self):
		return struct.calcsize(self.binaryFormat)
	
	def load(self, file):
		tmpData = file.read(struct.calcsize(self.binaryFormat))
		data = struct.unpack(self.binaryFormat, tmpData)
		self.start = data[0]
		self.length = data[1]
		return self
		
	def save(self, file):
		file.write(struct.pack(self.binaryFormat, \
			self.start, \
			self.length, \
		))
	
	def dump(self, log):
		log.info("zymlump:")
		log.info("  start: %i", self.start)
		log.info("  length: %i", self.length)
		log.info("")

class zymbone(object):
	__slots__ = 'name', 'flags', 'parent', 'binaryFormat'
	
	binaryFormat="!32sii"
	
	def __init__(self):
		self.name = ""
		self.flags =  0
		self.parent =  0
		
	def getsize(self):
		return struct.calcsize(self.binaryFormat)

	def load(self, file):
		tmpData = file.read(struct.calcsize(self.binaryFormat))
		data = struct.unpack(self.binaryFormat, tmpData)
		self.name = stripnull(data[0], 32)
		self.flags = data[1]
		self.parent = data[2]
		return self

	def save(self, file):
		file.write(struct.pack(self.binaryFormat, \
			self.name, \
			self.flags, \
			self.parent, \
		))

	def dump(self, log):
		log.info("zymbone:")
		log.info("  name: %s", self.name)
		log.info("  flags: %i", self.flags)
		log.info("  parent: %i", self.parent)
		log.info("")

class zymscene(object):
	__slots__ = 'name', 'mins', 'maxs', 'radius', \
		'framerate', 'flags', 'start', 'length', 'binaryFormat'
	
	binaryFormat="!32s3f3fffiii"
	
	def __init__(self):
		self.name = ""
		self.mins = [0.0, 0.0, 0.0]
		self.maxs = [0.0, 0.0, 0.0]
		self.radius = 0.0
		self.framerate =  0.0
		self.flags =  0
		self.start =  0
		self.length =  0
		
	def getsize(self):
		return struct.calcsize(self.binaryFormat)

	def load(self, file):
		tmpData = file.read(struct.calcsize(self.binaryFormat))
		data = struct.unpack(self.binaryFormat, tmpData)
		self.name = stripnull(data[0], 32)
		self.mins[0] = data[1]
		self.mins[1] = data[2]
		self.mins[2] = data[3]
		self.maxs[0] = data[4]
		self.maxs[1] = data[5]
		self.maxs[2] = data[6]
		self.radius = data[7]
		self.framerate = data[8]
		self.flags = data[9]
		self.start = data[10]
		self.length = data[11]
		return self

	def save(self, file):
		file.write(struct.pack(self.binaryFormat, \
			self.name, \
			self.mins[0], \
			self.mins[1], \
			self.mins[2], \
			self.maxs[0], \
			self.maxs[1], \
			self.maxs[2], \
			self.radius, \
			self.framerate, \
			self.flags, \
			self.start, \
			self.length, \
		))

	def dump(self, log):
		log.info("zymscene:")
		log.info("  name: %s", self.name)
		log.info("  mins: %s", self.mins)
		log.info("  maxs: %s", self.maxs)
		log.info("  radius: %f", self.radius)
		log.info("  framerate: %f", self.framerate)
		log.info("  flags: %i", self.flags)
		log.info("  start: %i", self.start)
		log.info("  length: %i", self.length)
		log.info("")

class zymvertex(object):
	__slots__ = 'bonenum', 'origin', 'binaryFormat'

	binaryFormat = "!i3f"

	def __init__(self):
		self.bonenum = 0
		self.origin = [0.0, 0.0, 0.0]

	def getsize(self):
		return struct.calcsize(self.binaryFormat)

	def load(self, file):
		tmpData = file.read(struct.calcsize(self.binaryFormat))
		data = struct.unpack(self.binaryFormat, tmpData)
		self.bonenum = data[0]
		self.origin[0] = data[1]
		self.origin[1] = data[2]
		self.origin[2] = data[3]
		return self

	def save(self, file):
		file.write(struct.pack(self.binaryFormat, \
			self.bonenum, \
			self.origin[0], \
			self.origin[1], \
			self.origin[2] \
		))

	def dump(self):
		log.info("zymvertex:")
		log.info("  bonenum: %i", self.bonenum)
		log.info("  origin: %s", self.origin)
		log.info("")

class zymtype1(object):
	__slots__ = \
		'id', 'type', 'filesize', 'mins', 'maxs', 'radius', \
		'numverts', 'numtris', 'numshaders', 'numbones', 'numscenes', \
		'lump_scenes', 'lump_poses', 'lump_bones', 'lump_vertbonecounts', \
		'lump_verts', 'lump_texcoords', 'lump_render', 'lump_shaders', \
		'lump_trizone', \
		'scenes', 'poses', 'bones', 'vertbonecounts', 'verts', 'texcoords', \
		'tris', 'shaders', 'trizones', \
		'tricount','numposes',\
		'binaryFormat'

	binaryFormat="!12sii3f3ff5i"

	def __init__(self):
		#header
		self.id = ""
		self.type = 0 #0 (vertex morph) 1 (skeletal pose) or 2 (skeletal scripted)
		self.filesize = 0
		self.mins = [0.0, 0.0, 0.0]
		self.maxs = [0.0, 0.0, 0.0]
		self.radius = 0.0
		self.numverts = 0
		self.numtris = 0
		self.numshaders = 0
		self.numbones = 0
		self.numscenes = 0
		self.lump_scenes = zymlump() #zymscene_t scene[numscenes];
		self.lump_poses = zymlump() #float pose[numposes][numbones][3][4];
		self.lump_bones = zymlump() #zymbone_t bone[numbones];
		self.lump_vertbonecounts = zymlump() #int vertbonecounts[numvertices];
		self.lump_verts = zymlump() #zymvertex_t vert[numvertices];
		self.lump_texcoords = zymlump() #float texcoords[numvertices][2]
		self.lump_render = zymlump() #int renderlist[rendersize];
		self.lump_shaders = zymlump() #char shadername[numshaders][32];
		self.lump_trizone = zymlump() #byte trizone[numtris];
		
		#data
		self.scenes = []
		self.poses = []
		self.bones = []
		self.vertbonecounts = []
		self.verts = []
		self.texcoords = []
		self.tris = []
		self.shaders = []
		self.trizones = []
		
		#helper
		self.tricount = 0
		self.numposes = 0
		

	def load(self, file, log):
		tmpData = file.read(struct.calcsize(self.binaryFormat))
		data = struct.unpack(self.binaryFormat, tmpData)
		self.id = stripnull(data[0],12)
		self.type = data[1]
		self.filesize = data[2]
		self.mins[0] = data[3]
		self.mins[1] = data[4]
		self.mins[2] = data[5]
		self.maxs[0] = data[6]
		self.maxs[1] = data[7]
		self.maxs[2] = data[8]
		self.radius = data[9]
		self.numverts = data[10]
		self.numtris = data[11]
		self.numshaders = data[12]
		self.numbones = data[13]
		self.numscenes = data[14]
		
		self.lump_scenes.load(file)
		self.lump_poses.load(file)
		self.lump_bones.load(file)
		self.lump_vertbonecounts.load(file)
		self.lump_verts.load(file)
		self.lump_texcoords.load(file)
		self.lump_render.load(file)
		self.lump_shaders.load(file)
		self.lump_trizone.load(file)
		
		file.seek(self.lump_scenes.start, 0)
		#read scenes
		for i in xrange(self.numscenes):
			self.scenes.append(zymscene())
			self.scenes[i].load(file)
			self.numposes += self.scenes[i].length
			#self.scenes[i].dump(log)
		
		file.seek(self.lump_poses.start, 0)
		#read poses
		for i in xrange(self.numposes):
			for j in xrange(self.numbones):
				data = struct.unpack("!12f", file.read(struct.calcsize("!12f")))
				self.poses.append( [data[0],data[1], data[2], data[3], data[4],\
					data[5], data[6], data[7],data[8], data[9], data[10],\
					data[11]] )
		
		file.seek(self.lump_bones.start, 0)
		#read bones
		for i in xrange(self.numbones):
			self.bones.append(zymbone())
			self.bones[i].load(file)
			#self.bones[i].dump(log)
		
		file.seek(self.lump_vertbonecounts.start, 0)
		#read vertbonecounts
		for i in xrange(self.numverts):
			data = struct.unpack("!i", file.read(struct.calcsize("!i")))
			self.vertbonecounts.append(data[0])
		
		file.seek(self.lump_verts.start, 0)
		#read verts
		for i in xrange(self.numverts):
			self.verts.append(zymvertex())
			self.verts[i].load(file)
		
		file.seek(self.lump_texcoords.start, 0)
		#read texcoords
		for i in xrange(self.numverts):
			data = struct.unpack("!ff", file.read(struct.calcsize("!ff")))
			self.texcoords.append( [data[0], data[1]] )
		
		file.seek(self.lump_render.start, 0)
		#read render
		#render[0] is tricount ,also something to do with trizone
		file.read(struct.calcsize("!i"))
		for i in xrange(self.numtris):
			data = struct.unpack("!3i", file.read(struct.calcsize("!3i")))
			self.tris.append( [data[0], data[1], data[2]] )
		
		file.seek(self.lump_shaders.start, 0)
		#read shaders
		for i in xrange(self.numshaders):
			data = struct.unpack("!32s", file.read(struct.calcsize("!32s")))
			self.shaders.append(stripnull(data[0], 32))
			#self.scenes[i].dump(log)
		
		file.seek(self.lump_trizone.start, 0)
		#read trizones
		for i in xrange(self.numtris):
			data = struct.unpack("!b", file.read(struct.calcsize("!b")))
			self.trizones.append(data[0])
		

	def getsize(self):
		self.lump_scenes.start = struct.calcsize(self.binaryFormat) + 9*self.lump_scenes.getsize()
		self.lump_scenes.length = self.numscenes * self.scenes[0].getsize()
		
		self.lump_poses.start = self.lump_scenes.start + self.lump_scenes.length
		self.lump_poses.length = self.numposes * self.numbones * struct.calcsize("!12f")
		
		self.lump_bones.start = self.lump_poses.start + self.lump_poses.length
		self.lump_bones.length = self.numbones * self.bones[0].getsize()
		
		self.lump_vertbonecounts.start = self.lump_bones.start + self.lump_bones.length
		self.lump_vertbonecounts.length = self.numverts * struct.calcsize("!i")
		
		self.lump_verts.start = self.lump_vertbonecounts.start + self.lump_vertbonecounts.length
		self.lump_verts.length = self.numverts * self.verts[0].getsize()
		
		self.lump_texcoords.start = self.lump_verts.start + self.lump_verts.length
		self.lump_texcoords.length = self.numverts * struct.calcsize("!ff")
		
		self.lump_render.start = self.lump_texcoords.start + self.lump_texcoords.length
		self.lump_render.length = struct.calcsize("!i") + self.numtris * struct.calcsize("!3i")
		
		self.lump_shaders.start = self.lump_render.start + self.lump_render.length
		self.lump_shaders.length = self.numshaders * struct.calcsize("!32s")
		
		self.lump_trizone.start = self.lump_shaders.start + self.lump_shaders.length
		self.lump_trizone.length = self.numtris * struct.calcsize("!b")
		
		self.filesize = self.lump_trizone.start + self.lump_trizone.length
	

	def save(self, file):
		self.getsize()
		
		file.write(struct.pack(self.binaryFormat, \
			self.id, \
			self.type, \
			self.filesize, \
			self.mins[0], \
			self.mins[1], \
			self.mins[2], \
			self.mins[0], \
			self.maxs[1], \
			self.maxs[2], \
			self.radius, \
			self.numverts, \
			self.numtris, \
			self.numshaders, \
			self.numbones, \
			self.numscenes, \
		))
		
		self.lump_scenes.save(file)
		self.lump_poses.save(file)
		self.lump_bones.save(file)
		self.lump_vertbonecounts.save(file)
		self.lump_verts.save(file)
		self.lump_texcoords.save(file)
		self.lump_render.save(file)
		self.lump_shaders.save(file)
		self.lump_trizone.save(file)
		
		#write scenes
		for i in xrange(self.numscenes):
			self.scenes[i].save(file)
		
		#write poses
		for i in xrange(self.numposes):
			for j in xrange(self.numbones):
				file.write(struct.pack("!12f", \
					self.poses[i*self.numbones+j][0], \
					self.poses[i*self.numbones+j][1], \
					self.poses[i*self.numbones+j][2], \
					self.poses[i*self.numbones+j][3], \
					self.poses[i*self.numbones+j][4], \
					self.poses[i*self.numbones+j][5], \
					self.poses[i*self.numbones+j][6], \
					self.poses[i*self.numbones+j][7], \
					self.poses[i*self.numbones+j][8], \
					self.poses[i*self.numbones+j][9], \
					self.poses[i*self.numbones+j][10], \
					self.poses[i*self.numbones+j][11] \
				))
		
		#write bones
		for i in xrange(self.numbones):
			self.bones[i].save(file)
		
		#write vertbonecounts 1,1,1,1,.....
		for i in xrange(self.numverts):
			file.write(struct.pack("!i", \
				self.vertbonecounts[i] \
			))
		
		#write verts
		for i in xrange(self.numverts):
			self.verts[i].save(file)
		
		#write texcoords
		for i in xrange(self.numverts):
			file.write(struct.pack("!ff", \
				self.texcoords[i][0], \
				self.texcoords[i][1] \
			))
		
		#write render[or tris]
		file.write(struct.pack("!i", self.numtris))
		for i in xrange(self.numtris):
			file.write(struct.pack("!3i", \
				self.tris[i][0], \
				self.tris[i][1], \
				self.tris[i][2] \
			))
		
		
		#write shaders
		for i in xrange(self.numshaders):
			file.write(struct.pack("!32s", \
				self.shaders[i] \
			))
		
		#read trizones
		for i in xrange(self.numtris):
			file.write(struct.pack("!b", \
				self.trizones[i] \
			))


	def dump(self, log):
		log.info("zymtype1:")
		log.info("  id: %s", self.id)
		log.info("  type: %i", self.type)
		log.info("  filesize: %i", self.filesize)
		log.info("  mins: %s", self.mins)
		log.info("  maxs: %s", self.maxs)
		log.info("  radius: %s", self.radius)
		log.info("  numverts: %i", self.numverts)
		log.info("  numtris: %i", self.numtris)
		log.info("  numshaders: %i", self.numshaders)
		log.info("  numbones: %i", self.numbones)
		log.info("  numscenes: %i", self.numscenes)
		
		log.info("")
		log.info("lump_scenes")
		self.lump_scenes.dump(log)
		log.info("lump_poses")
		self.lump_poses.dump(log)
		log.info("lump_bones")
		self.lump_bones.dump(log)
		log.info("lump_vertbonecounts")
		self.lump_vertbonecounts.dump(log)
		log.info("lump_verts")
		self.lump_verts.dump(log)
		log.info("lump_texcoords")
		self.lump_texcoords.dump(log)
		log.info("lump_render")
		self.lump_render.dump(log)
		log.info("lump_shaders")
		self.lump_shaders.dump(log)
		log.info("lump_trizone")
		self.lump_trizone.dump(log)
		
		for i in xrange(self.numscenes):
			self.scenes[i].dump(log)
		
		#for i in xrange(self.numbones):
		#	self.bones[i].dump(log)
		
		#for i in xrange(self.numverts):
		#log.info("vertbonecount: %s", self.vertbonecounts)
		#log.info("vert: %s", self.verts)
		
		#log.info("poses: %s", self.poses)
		#log.info("trizone: %s", self.trizones)
		#log.info("vertbonecount: %s", self.vertbonecounts)
		
		for i in xrange(self.numshaders):
			log.info("shader: %s", self.shaders[i])
		log.info("")
