#!BPY

"""
Name: 'Zymotic (.zym)'
Blender: 245
Group: 'Export'
Tooltip: 'Export to Zymotic file format. (.zym)'
"""

__author__ = "f0rqu3"
__url__ = ("http://example.org")
__version__ = "0.3 2008-01-05"

__bpydoc__ = """\
This script exports a nexuiz zymotic type1 model file (ZYM)
by f0rqu3

Supported:<br>
    none

Missing:<br>
    all

Known issues:<br>
    dunno

Notes:<br>
    TODO
"""

import zym
from zym import *

import zym_shared
from zym_shared import *

import Blender

import BPyMesh

log = Logger("zym_import_log")

class BlenderGui:
	def __init__(self):
		text = ["A log has been written to a blender text window.",
			"Change this window type to a text window and", 
			"you will be able to select the file zym_export_log." ]
		
		text+=["Parsed in %i seconds"%(Blender.sys.time()-log.start)]
		
		if log.has_critical:
			text += ["There were critical errors!!!!"]
		elif log.has_errors:
			text += ["There were errors!"]
		elif log.has_warnings:
			text += ["There were warnings"]
		
		text.reverse()
		self.msg = text
		
		Blender.Draw.Register(self.gui, self.event, self.button_event)
		
	def gui(self,):
		quitbutton = Blender.Draw.Button("Exit", 1, 0, 0, 100, 20, "Close Window")
		
		y = 35
		
		#fix me
		for line in self.msg:
			Blender.BGL.glRasterPos2i(10,y)
			Blender.Draw.Text(line)
			y+=15
			
	def event(self,evt, val):
		if evt == Blender.Draw.ESCKEY:
			Blender.Draw.Exit()
			return
	
	def button_event(self,evt):
		if evt == 1:
			Blender.Draw.Exit()
			return


def ApplyTransform(vert, matrix):
	return vert * matrix


def UpdateFrameBounds(v, f):
	for i in xrange(3):
		f.mins[i] = min(v[i], f.mins[i])
	for i in xrange(3):
		f.maxs[i] = max(v[i], f.maxs[i])


def UpdateFrameRadius(f):
	f.radius = RadiusFromBounds(f.mins, f.maxs)


def ProcessSurface(scene, blenderObject, md3, pathName, modelName):
	# because md3 doesnt suppoort faceUVs like blender, we need to duplicate
	# any vertex that has multiple uv coords

	vertDict = {}
	indexDict = {} # maps a vertex index to the revised index after duplicating to account for uv
	vertList = [] # list of vertices ordered by revised index
	numVerts = 0
	uvList = [] # list of tex coords ordered by revised index
	faceList = [] # list of faces (they index into vertList)
	numFaces = 0

	scene.makeCurrent()
	Blender.Set("curframe", 1)
	Blender.Window.Redraw()

	# get the object (not just name) and the Mesh, not NMesh
	mesh = blenderObject.getData(False, True)
	matrix = blenderObject.getMatrix('worldspace')

	surf = md3Surface()
	surf.numFrames = md3.numFrames
	surf.name = blenderObject.getName()
	surf.ident = MD3_IDENT
	
	# create shader for surface
	surf.shaders.append(md3Shader())
	surf.numShaders += 1
	surf.shaders[0].index = 0
	
	log.info("Materials: %s", mesh.materials)
	# :P
	if not mesh.materials:
		surf.shaders[0].name = pathName + blenderObject.name
	else:
		surf.shaders[0].name = pathName + mesh.materials[0].name

	# process each face in the mesh
	for face in mesh.faces:
		if mesh.faceUV == True:
			face_uv = face.uv
		tris_in_this_face = []  #to handle quads and up...
		
		# this makes a list of indices for each tri in this face. a quad will be [[0,1,1],[0,2,3]]
		for vi in xrange(1, len(face)-1):
			tris_in_this_face.append([0, vi, vi + 1])
		
		# loop across each tri in the face, then each vertex in the tri
		for this_tri in tris_in_this_face:
			numFaces += 1
			tri = md3Triangle()
			tri_ind = 0
			for i in this_tri:
				# get the vertex index, coords and uv coords
				index = face.v[i].index
				v = face.v[i].co
				if mesh.faceUV == True:
					uv = tuple(face_uv[i])
				elif mesh.vertexUV:
					uv = (face.v[i].uvco[0], face.v[i].uvco[1])
				else:
					uv = (0.0, 0.0) # handle case with no tex coords	

				
				if vertDict.has_key((index, uv)):
					# if we've seen this exact vertex before, simply add it
					# to the tris list of vertex indices
					tri.indexes[tri_ind] = vertDict[(index, uv)]
				else:
					# havent seen this tri before 
					# (or its uv coord is different, so we need to duplicate it)
					
					vertDict[(index, uv)] = numVerts
					
					# put the uv coord into the list
					# (uv coord are directly related to each vertex)
					tex = md3TexCoord()
					tex.u = uv[0]
					tex.v = uv[1]
					uvList.append(tex)

					tri.indexes[tri_ind] = numVerts

					# now because we have created a new index, 
					# we need a way to link it to the index that
					# blender returns for NMVert.index
					if indexDict.has_key(index):
						# already there - each of the entries against 
						# this key represents  the same vertex with a
						# different uv value
						ilist = indexDict[index]
						ilist.append(numVerts)
						indexDict[index] = ilist
					else:
						# this is a new one
						indexDict[index] = [numVerts]

					numVerts += 1
				tri_ind +=1
			faceList.append(tri)

	# we're done with faces and uv coords
	surf.uv.extend(uvList)

	surf.triangles.extend(faceList)

	surf.numTriangles = len(faceList)
	surf.numVerts = numVerts

	# now vertices are stored as frames -
	# all vertices for frame 1, all vertices for frame 2...., all vertices for frame n
	# so we need to iterate across blender's frames, and copy out each vertex
	for	frameNum in xrange(1, md3.numFrames + 1):
		Blender.Set("curframe", frameNum)
		Blender.Window.Redraw()

		m = BPyMesh.getMeshFromObject(blenderObject)
		m.transform(matrix)

		vlist = [0] * numVerts
		for vertex in m.verts:
			try:
				vindices = indexDict[vertex.index]
			except:
				log.warning("Found a vertex in %s that is not part of a face", blenderObject.name)
				continue

			vTx = vertex.co
			nTx = vertex.no
			UpdateFrameBounds(vTx, md3.frames[frameNum - 1])
			vert = md3Vert()
			#vert.xyz = vertex.co[0:3]
			#vert.normal = vert.Encode(vertex.no[0:3])
			vert.xyz = vTx[0:3]
			vert.normal = Encode(nTx[0:3])
			for ind in vindices:  # apply the position to all the duplicated vertices
				vlist[ind] = vert

		UpdateFrameRadius(md3.frames[frameNum - 1])

		surf.verts.extend(vlist)

	surf.Dump(log)
	md3.surfaces.append(surf)
	md3.numSurfaces += 1


def Export(fileName):
	if(fileName.find('.zym', -4) <= 0):
		fileName += '.zym'
	
	pathName = StripGamePath(StripModel(fileName))
	
	# ask for a new shader path
	text = Blender.Draw.Create(pathName)
	block = [ ("",text , 0, MAX_QPATH, "Ex: models/players/human_base/") ]
	retval = Blender.Draw.PupBlock("Shader Path :", block)
	#changed
	if retval: pathName = text.val
	
	#log starts here
	log.start = Blender.sys.time()
	log.info("Starting ...")
	
	log.info("Exporting ZYM format to: %s", fileName)
	
	log.info("Shader path name: %s", pathName)
	
	modelName = StripExtension(StripPath(fileName))
	log.info("Model name: %s", modelName)
	
	zym = zymtype1()
	zym.id = "ZYMOTICMODEL"
	zym.type = 1
	
	#zym.filesize = 10
	zym.mins = [-1.0, -1.0, -1.0]
	zym.maxs = [1.0, 1.0, 1.0]
	zym.radius = 1.0
	zym.numverts = 3
	zym.numtris = 1
	zym.numshaders = 1
	zym.numbones = 1
	zym.numscenes = 1
	
	zym.numposes = 1
	
	
	zscene = zymscene()
	zscene.name = "plox"
	zscene.mins = [1.0, 1.0, 1.0]
	zscene.maxs = [3.0, 3.0, 3.0]
	zscene.radius = 44.0
	zscene.framerate =  30.0
	zscene.flags =  13
	zscene.start =  0
	zscene.length =  1
	
	zym.scenes.append(zscene)
	
	zym.poses.append([1,0,0,0, 0,1,0,0, 0,0,1,5])
	
	zbone = zymbone()
	zbone.name= "blahhh"
	zbone.flags = 111
	zbone.parent = -1
	
	zym.bones.append(zbone)
	
	zym.vertbonecounts.append(1)
	zym.vertbonecounts.append(1)
	zym.vertbonecounts.append(1)
	
	zvert1 = zymvertex()
	zvert1.bonenum = 0
	zvert1.origin = [5,0,0]
	zym.verts.append(zvert1)
	
	zvert2 = zymvertex()
	zvert2.bonenum = 0
	zvert2.origin = [0,10,0]
	zym.verts.append(zvert2)
	
	zvert3 = zymvertex()
	zvert3.bonenum = 0
	zvert3.origin = [0,0,0]
	zym.verts.append(zvert3)
	
	zym.texcoords.append([1,0])
	zym.texcoords.append([0,1])
	zym.texcoords.append([0,0])
	
	zym.tris.append( [0, 1, 2] )
	
	zym.shaders.append("arghhh")
	
	zym.trizones.append(0)
	zym.trizones.append(0)
	zym.trizones.append(0)

	"""
	# get the scene
	scene = Blender.Scene.GetCurrent()
	context = scene.getRenderingContext()

	scene.makeCurrent()
	md3.numposes = Blender.Get("curframe")
	Blender.Set("curframe", 1)

	
	# create a bunch of blank frames, they'll be filled in by 'ProcessSurface'
	for i in xrange(1, md3.numFrames + 1):
		frame = md3Frame()
		frame.name = "frame_" + str(i)
		md3.frames.append(frame)

	# export all selected objects
	objlist = Blender.Object.GetSelected()

	# process each object for the export
	for obj in objlist:
		# check if it's a mesh object
		if obj.getType() == "Mesh":
			log.info("Processing surface: %s", obj.name)
			if len(md3.surfaces) == MD3_MAX_SURFACES:
				log.warning("Hit md3 limit (%i) for number of surfaces, skipping ...", MD3_MAX_SURFACES, obj.getName())
			else:
				ProcessSurface(scene, obj, md3, pathName, modelName)
		elif obj.getType() == "Empty":   # for tags, we just put em in a list so we can process them all together
			if obj.name[0:4] == "tag_":
				log.info("Processing tag: %s", obj.name)
				tagList.append(obj)
				md3.numTags += 1
		else:
			log.info("Skipping object: %s", obj.name)

	
	# work out the transforms for the tags for each frame of the export
	for i in xrange(1, md3.numFrames + 1):
		
		# needed to update IPO's value, but probably not the best way for that...
		scene.makeCurrent()
		Blender.Set("curframe", i)
		Blender.Window.Redraw()
		for tag in tagList:
			t = md3Tag()
			matrix = tag.getMatrix('worldspace')
			t.origin[0] = matrix[3][0]
			t.origin[1] = matrix[3][1]
			t.origin[2] = matrix[3][2]
				
			t.axis[0] = matrix[0][0]
			t.axis[1] = matrix[0][1]
			t.axis[2] = matrix[0][2]
				
			t.axis[3] = matrix[1][0]
			t.axis[4] = matrix[1][1]
			t.axis[5] = matrix[1][2]
				
			t.axis[6] = matrix[2][0]
			t.axis[7] = matrix[2][1]
			t.axis[8] = matrix[2][2]
			t.name = tag.name
			#t.Dump(log)
			md3.tags.append(t)
	"""
	# export!
	file = open(fileName, "wb")
	zym.save(file)
	file.close()
	zym.dump(log)

def FileSelectorCallback(fileName):
	Export(fileName)
	
	BlenderGui()

Blender.Window.FileSelector(FileSelectorCallback, "Export Nexuiz ZYM", Blender.sys.makename(ext='.zym'))
