#!BPY

"""
Name: 'Zymotic (.zym)'
Blender: 245
Group: 'Import'
Tooltip: 'Import from Zymotic file format. (.zym)'
"""

__author__ = "f0rqu3"
__url__ = ("http://example.org")
__version__ = "0.3 2008-01-05"

__bpydoc__ = """\
This script imports a nexuiz zymotic type1 model file (ZYM)
by f0rqu3

Supported:<br>
    animation mesh

Missing:<br>
    bone length
    some animations look incorretc?

Known issues:<br>
    dunno

Notes:<br>
    TODO
"""

import zym
from zym import *

import zym_shared
from zym_shared import *

log = Logger("zym_import_log")

class BlenderGui:
	def __init__(self):
		text = ["A log has been written to a blender text window.",
			"Change this window type to a text window.", 
			"You will be able to select the file %s." % log.name ]
		
		text+=["Parsed in %i seconds"%(Blender.sys.time() - log.start)]
		
		if log.has_critical:
			text += ["There were critical errors!!!!"]
		elif log.has_errors:
			text += ["There were errors!"]
		elif log.has_warnings:
			text += ["There were warnings"]
		
		text.reverse()
		self.msg = text
		
		Blender.Draw.Register(self.gui, self.event, self.button_event)
		
	def gui(self,):
		quitbutton = Blender.Draw.Button("Exit", 1, 0, 0, 100, 20, "Close Window")
		
		y = 35
		
		#fix me
		for line in self.msg:
			Blender.BGL.glRasterPos2i(10,y)
			Blender.Draw.Text(line)
			y+=15
			
	def event(self,evt, val):
		if evt == Blender.Draw.ESCKEY:
			Blender.Draw.Exit()
			return
	
	def button_event(self,evt):
		if evt == 1:
			Blender.Draw.Exit()
			return

def Import(fileName):
	#log starts here
	log.start = Blender.sys.time()
	log.info("Starting ...")
	
	log.info("Importing ZYM model: %s", fileName)
	
	pathName = StripGamePath(StripModel(fileName))
	log.info("Shader path name: %s", pathName)
	
	modelName = StripExtension(StripPath(fileName))
	log.info("Model name: %s", modelName)
	
	# read the file in
	file = open(fileName,"rb")
	zym = zymtype1()
	zym.load(file, log)
	zym.dump(log)
	file.close()
	
	scene = Blender.Scene.GetCurrent()
	
	# create a new mesh
	mesh = Blender.Mesh.New(modelName)
	
	# mesh has vertex uv :/
	mesh.vertexUV = True
	
	boneposes = []
	#generate initial boneposes
	for i in xrange(zym.numbones):
		boneposes.append( zym.poses[i] )
	for i in xrange(zym.numbones):
		if(zym.bones[i].parent != -1):
			boneposes[i] = contrans( boneposes[zym.bones[i].parent], boneposes[i] )
	
	# create the verts
	#mesh.verts.extend( [ zym.verts[i].origin for i in xrange(zym.numverts)] )
	mesh.verts.extend( [\
		vertorg(zym.verts[i].origin, boneposes[zym.verts[i].bonenum])\
		for i in xrange(zym.numverts)] )
	
	# create uv
	for i in xrange(len(mesh.verts)):
		mesh.verts[i].uvco=Blender.Mathutils.Vector(zym.texcoords[i][0], zym.texcoords[i][1])
	
	# create the faces
	mesh.faces.extend( [zym.tris[i] for i in xrange(zym.numtris)] )
	
	# vertex uv to face uv
	mesh.faceUV=True
	for f in mesh.faces:
		f.uv = [v.uvco for v in f.verts]
		f.smooth=1
	
	# calculate normals
	mesh.calcNormals()
	
	# create materials for surface
	# TODO : omg images
	for i in xrange(zym.numshaders):
		
		# create new material if necessary
		matName = StripExtension(StripPath(zym.shaders[i]))
		if matName == "" :
			matName = surface.name + "_mat"
		
		try:
			mat = Blender.Material.Get(matName)
		except NameError:
			log.info("Creating new material: %s", matName)
			mat = Blender.Material.New(matName)
			# create new texture
			texture = Blender.Texture.New(matName)
			texture.setType('Image')
			# try .tga by default
			#imageName = StripExtension(GAMEDIR + surface.shaders[i].name) + '.tga'
			imageName = StripExtension(pathName+matName)
			"""
			try:
				image = Blender.Image.Load(imageName)
				texture.image = image
			except IOError:
				try:
					imageName = StripExtension(imageName) + '.png'
					image = Blender.Image.Load(imageName)
					texture.image = image
				except IOError:
					try:
						imageName = StripExtension(imageName) + '.jpg'
						image = Blender.Image.Load(imageName)
						texture.image = image
					except IOError:
						log.warning("Unable to load image for %s", surface.name)
			"""
			# texture to material
			mat.setTexture(0, texture, Blender.Texture.TexCo.UV, Blender.Texture.MapTo.COL)

		# append material to the mesh's list of materials
		mesh.materials+=[mat]
		#mesh.update()
	
	# add object
	meshObject= Blender.Object.New('Mesh',modelName)
	meshObject.link(mesh)
	"""
		if surface.numFrames > 1 :
			# animate the verts through keyframe animation
			for i in xrange(surface.numFrames):
				
				# update the vertices
				for j in xrange(surface.numVerts):
					xyz=Blender.Mathutils.Vector(surface.verts[(i * surface.numVerts) + j].xyz)
					#normal=Mathutils.Vector(Decode(surface.verts[(i * surface.numVerts) + j].normal))
					#mesh.verts[j].no = normal
					mesh.verts[j].co = xyz
					
				meshObject.insertShapeKey()

			meshKey = mesh.key
			meshKey.ipo = Blender.Ipo.New('Key', surface.name + "_ipo")
			
			index = 1
			for curveName in meshKey.ipo.curveConsts :
				#print curveName
				meshKey.ipo.addCurve(curveName)
				meshKey.ipo[curveName].interpolation=Blender.IpoCurve.InterpTypes.CONST
				meshKey.ipo[curveName].addBezier((0,0))
				meshKey.ipo[curveName].addBezier((index,1))
				meshKey.ipo[curveName].addBezier((index+1,0))
				index+=1
		
		# select all and remove doubles
		#mesh.sel=1
		#mesh.remDoubles(0.0)
	"""
	
	#create arm
	armObject = Blender.Object.New('Armature', modelName+"_ArmOB")
	armData = Blender.Armature.Armature(modelName+"_Arm")
	armObject.link(armData)
	
	armData.vertexGroups = True 
	#armData.drawAxes = True 
	armData.drawType = Blender.Armature.STICK #set the draw type
	armData.makeEditable() 
	
	
	#bone lengths from boneposes
	bonelen = [0]*zym.numbones
	subbones = [0]*zym.numbones
	for i in xrange(zym.numbones):
		p = boneposes[i]
		if(zym.bones[i].parent != -1):
			parentnum = zym.bones[i].parent
			subbones[parentnum] += 1
			pos1 = Blender.Mathutils.Vector(poseloc(p))
			pos2 = Blender.Mathutils.Vector(poseloc( boneposes[ parentnum ]))
			bonelen[ parentnum ] += (pos2 - pos1).length
	for i in xrange(zym.numbones):
		if( subbones[i] != 0):
			bonelen[i] /= subbones[i]
			if(bonelen[i]==0):
				bonelen[i] = 1
		else:
			bonelen[i] = 1
	
	
	#bone placement from boneposes (generate from pose 0)
	for i in xrange(zym.numbones):
		blenderbone = Blender.Armature.Editbone()
		p = boneposes[i]# origin
		
		if(zym.bones[i].parent != -1):
			parentbone = armData.bones[ zym.bones[ zym.bones[i].parent ].name ]
			blenderbone.parent = parentbone
		
		blenderbone.head = Blender.Mathutils.Vector(poseloc(p))
		blenderbone.tail = Blender.Mathutils.Vector(poseloc(p)) + \
			Blender.Mathutils.Vector([bonelen[i],0,0]) #1 is length
		
		#editbone.matrix also have scale info
		#we need to det?normalize:identity
		matrix = Blender.Mathutils.Matrix([-p[1],-p[5],-p[9]],[p[0],p[4],p[8]],[p[2],p[6],p[10]])
		lenx = matrix[0].length
		leny = matrix[1].length
		lenz = matrix[2].length
		#print zym.bones[i].name,lenx,leny,lenz,matrix.determinant()
		if(lenx+leny+lenz == 0):
			matrix.identity()
		else:
			matrix[0] *= (1.0/lenx)
			matrix[1] *= (1.0/leny)
			matrix[2] *= (1.0/lenz)
		#print zym.bones[i].name,matrix[0].length,matrix[1].length,matrix[2].length,matrix.determinant()
		blenderbone.matrix = matrix
		
		armData.bones[ zym.bones[i].name ] = blenderbone
	
	armData.update()
	
	# add vertex groups
	# adding too much?
	for i in xrange(zym.numbones):
		mesh.addVertGroup(zym.bones[i].name)
	
	for i in xrange(zym.numverts):
		mesh.assignVertsToGroup(zym.bones[ zym.verts[i].bonenum ].name, [ i ],
														1.0, Blender.Mesh.AssignModes.ADD )
	
	#xray and parent
	armObject.xRay = True
	armObject.makeParent([meshObject])
	
	#add armature modifier
	mod = meshObject.modifiers.append(Blender.Modifier.Types.ARMATURE)
	mod[Blender.Modifier.Settings.OBJECT] = armObject
	
	#add to the scene
	scene.objects.link(meshObject)
	scene.objects.link(armObject) 
	scene.update(); 
	
	pose = armObject.getPose()
	"""
	action = armObject.getAction()
	if not action: # Add a pose action if we dont have one
		action = Blender.Armature.NLA.NewAction()
		action.setActive(armObject)
	
	#pose the bones
	for i in xrange(zym.numposes):
		for j in xrange(zym.numbones):
			m = zym.poses[i*zym.numbones+j]
			matrix = Blender.Mathutils.Matrix( [m[0],m[4],m[8]], [m[1],m[5],m[9]], [m[2],m[6],m[10]] )
			pose.bones[zym.bones[j].name].quat = matrix.toQuat()
			pose.bones[zym.bones[j].name].loc = Blender.Mathutils.Vector(poseorg(m))
			
			pose.bones[zym.bones[j].name].insertKey(armObject, i+1, [Blender.Object.Pose.ROT, Blender.Object.Pose.LOC], True)
	"""
	
	#action = armObject.getAction()
	#if not action: # Add a pose action if we dont have one
	#	action = Blender.Armature.NLA.NewAction()
	#	action.setActive(armObject)
	
	#pose the bones
	
	for s in xrange(zym.numscenes):
		scene = zym.scenes[s]
		action = Blender.Armature.NLA.NewAction(scene.name)
		action.setActive(armObject)
		armObject.actionStrips.append(action)
		#p = zym.poses[scene.start]
		for i in xrange(scene.start,scene.start+scene.length):
			for j in xrange(zym.numbones):
				p = zym.poses[j]
				m = zym.poses[i*zym.numbones+j]
				matrix1 = Blender.Mathutils.Matrix( [-m[1],-m[5],-m[9]], [m[0],m[4],m[8]], [m[2],m[6],m[10]] )
				matrix2 = Blender.Mathutils.Matrix( [-p[1],-p[5],-p[9]], [p[0],p[4],p[8]], [p[2],p[6],p[10]] )
				pose.bones[zym.bones[j].name].quat = Blender.Mathutils.DifferenceQuats( matrix2.toQuat() , matrix1.toQuat() )
				pose.bones[zym.bones[j].name].loc = Blender.Mathutils.Vector(poseloc(m)) - Blender.Mathutils.Vector(poseloc(p))
				
				pose.bones[zym.bones[j].name].insertKey(armObject, i-scene.start+1, [Blender.Object.Pose.ROT, Blender.Object.Pose.LOC], True)
	"""
	scene = zym.scenes[0]
	action = Blender.Armature.NLA.NewAction(scene.name)
	action.setActive(armObject)
	armObject.actionStrips.append(action)
	for i in xrange(0,zym.numposes):
		for j in xrange(zym.numbones):
			p = zym.poses[j]
			m = zym.poses[i*zym.numbones+j]
			matrix1 = Blender.Mathutils.Matrix( [-m[1],-m[5],-m[9]], [m[0],m[4],m[8]], [m[2],m[6],m[10]] )
			matrix2 = Blender.Mathutils.Matrix( [-p[1],-p[5],-p[9]], [p[0],p[4],p[8]], [p[2],p[6],p[10]] )
			pose.bones[zym.bones[j].name].quat = Blender.Mathutils.DifferenceQuats( matrix2.toQuat() , matrix1.toQuat() )
			pose.bones[zym.bones[j].name].loc = Blender.Mathutils.Vector(poseloc(m)) - Blender.Mathutils.Vector(poseloc(p))
			
			pose.bones[zym.bones[j].name].insertKey(armObject, i+1, [Blender.Object.Pose.ROT, Blender.Object.Pose.LOC], True)
	"""


def FileSelectorCallback(fileName):
	Import(fileName)
	
	BlenderGui()

Blender.Window.FileSelector(FileSelectorCallback, 'Import Nexuiz ZYM', Blender.sys.makename(ext='.zym'))
